from fastai.text import *
import sentencepiece as spm
from pathlib import Path

path = Path(__file__).parent


class LanguageTokenizer(BaseTokenizer):
    def __init__(self, lang: str):
        self.lang = lang
        self.sp = spm.SentencePieceProcessor()
        model_path = path/f'models/{lang}/tokenizer.model'
        self.sp.Load(str(model_path))

    def tokenizer(self, t: str) -> List[str]:
        return self.sp.EncodeAsPieces(t)

    def numericalize(self, t: str) -> List[int]:
        return self.sp.EncodeAsIds(t)

    def textify(self, ids: List[int]) -> str:
        return (''.join([self.sp.IdToPiece(id).replace('▁', ' ') for id in ids])).strip()

    def remove_foreign_tokens(self, t: str):
        local_pieces = []
        for i in self.sp.EncodeAsIds(t):
            local_pieces.append(self.sp.IdToPiece(i))
        return local_pieces


class AllLanguageTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class SanskritTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class BengaliTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class GujaratiTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class HindiTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class KannadaTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class MalyalamTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class MarathiTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class NepaliTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class OriyaTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class PanjabiTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class TamilTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)


class UrduTokenizer(LanguageTokenizer):
    def __init__(self, lang: str):
        LanguageTokenizer.__init__(self, lang)



from pathlib import Path

import aiohttp as aiohttp
import os

from .config import LanguageCodes, LMConfigs, AllLanguageConfig

all_language_codes = LanguageCodes()

path = Path(__file__).parent


async def download_file(url, dest, fname):
    if (dest/f'{fname}').exists(): return False
    os.makedirs(dest, exist_ok=True)
    print('Downloading Model. This might take time, depending on your internet connection. Please be patient.\n'
          'We\'ll only do this for the first time.')
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            data = await response.read()
            with open(dest/f'{fname}', 'wb') as f:
                f.write(data)
    return True


async def setup_language(language_code: str):
    lmconfig = LMConfigs(language_code)
    config = lmconfig.get_config()
    await download_file(config['lm_model_url'], path/'models'/f'{language_code}', config["lm_model_file_name"])
    await download_file(config['tokenizer_model_url'], path/'models'/f'{language_code}',
                        config["tokenizer_model_file_name"])
    print('Done!')
    return True


def verify_language(language_code: str):
    lmconfig = LMConfigs(language_code)
    config = lmconfig.get_config()
    if (path/'models'/f'{language_code}'/f'{config["lm_model_file_name"]}').exists() and \
            (path/'models'/f'{language_code}'/f'{config["tokenizer_model_file_name"]}').exists():
        return True
    else:
        return False


async def check_all_languages_identifying_model():
    config = AllLanguageConfig.get_config()
    if (path/'models'/'all'/f'{config["all_languages_identifying_model_name"]}').exists() and \
            (path/'models'/'all'/f'{config["all_languages_identifying_tokenizer_name"]}').exists():
        return True
    done = await download_file(config["all_languages_identifying_model_url"], path/'models'/'all',
                        config["all_languages_identifying_model_name"])
    done = await download_file(config["all_languages_identifying_tokenizer_url"], path/'models'/'all',
                        config["all_languages_identifying_tokenizer_name"])
    return done




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="inltk",
    version="0.7.1",
    author="Gaurav",
    author_email="contactgauravforwork@gmail.com",
    description="Natural Language Toolkit for Indian Languages (iNLTK)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/goru001/inltk",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux"
    ],
    dependency_links=[
        'http://download.pytorch.org/whl/cpu/torch-1.0.0-cp36-cp36m-linux_x86_64.whl'
    ],
    install_requires=[
        'aiohttp>=3.5.4',
        'async-timeout>=3.0.1',
        "Pillow",
        "beautifulsoup4",
        "bottleneck",
        "dataclasses;python_version<'3.7'",
        "fastprogress>=0.1.19",
        "matplotlib",
        "numexpr",
        "numpy>=1.15",
        "nvidia-ml-py3",
        "packaging",
        "pandas",
        "pynvx>=1.0.0;platform_system=='Darwin'",
        "pyyaml",
        "requests",
        "scipy",
        "spacy>=2.0.18",
        "typing",
        'fastai==1.0.57',
        "sentencepiece"
    ],
)
